package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.UserFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.Token;
import com.bwie.common.result.Result;

import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.zip.Inflater;

@Service
public class AuthServiceImpl implements AuthService {
    @Resource
    private UserFeign userFeign;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;

    @Override
    public Result authSendCode(String userTel) {
//        校验
        if (StringUtils.isBlank( userTel )){
            return Result.error(500,"请输入手机号");
        }
        Result<User> userResult = userFeign.userSendCode( userTel );
        User data = userResult.getData();
        if(data==null){
            return Result.error(500,"请输入正确的手机号");
        }
//        生成四位随机数做验证码
        int code = new Random().nextInt( 9000 ) + 1000;
        System.out.println("验证码为=========>"+code);
//        存Redis
        redisTemplate.opsForValue().set( "Code:"+userTel, JSONObject.toJSONString( code ),60, TimeUnit.MINUTES );
//        返回
        return Result.success(200,"发送成功 本次验证码为："+code);
    }



    @Override
    public Result authFindByuserTel(User user) {
//        校验
        if (StringUtils.isBlank( user.getUserTel() )){
            return Result.error(500,"请输入手机号");
        }
//校验
        if (StringUtils.isBlank( user.getCode() )){
            return Result.error(500,"请输入验证码");
        }
//        取Redis验证码
        String echeCode = redisTemplate.opsForValue().get( "Code:" + user.getUserTel() );
//        娶不到或不正确
        if (!echeCode.equals( user.getCode() )){
            return Result.error(500,"验证码不正确 请重试");
        }
        if (echeCode==null){
            return Result.error(500,"验证码过期 请重试");
        }
//查询库内数据
        Result<User> userResult = userFeign.userFindByuserTel( user );
        User data = userResult.getData();
        if(data==null){
            return Result.error(500,"该手机号未进行绑定 请先注册");
        }
//        生成令牌
        String replace = UUID.randomUUID().toString().replace( "-", "" );
        Map<String, Object> map = new HashMap<>();
        map.put( JwtConstants.USER_KEY,replace );
        String token = JwtUtils.createToken( map );

        redisTemplate.opsForValue().set( "token:"+replace,JSONObject.toJSONString( user ),60,TimeUnit.MINUTES );
//        存好
        Token token1 = new Token();
        token1.setToken( token );
        token1.setTime( "60分钟有效" );
//        返回
        return Result.success(token1);
    }
    @Override
    public Result info() {
        String token = request.getHeader( "token" );
        String userKey = JwtUtils.getUserKey( token );
        String s = redisTemplate.opsForValue().get( "token:" + userKey );
        User user = JSONObject.parseObject( s, User.class );
        return Result.success(user);
    }

}
