package com.bwie.gateway.filters;

import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.TimeUnit;


public class GateWayFilter implements GatewayFilter, Ordered {

    @Resource
    private IgnoreWhiteConfig ignoreWhiteConfig;

    @Resource
    private StringRedisTemplate redisTemplate;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        //获取请求白名单
        List<String> whites = ignoreWhiteConfig.getWhites();
        //通过exchange获取当前请求
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        //判断是否在白名单中
        if (StringUtils.matches(path,whites)){
            chain.filter(exchange);
        }
        //判断token是否过期
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        if (StringUtils.isBlank(token)){
            return GatewayUtils.errorResponse(exchange,"token不能为空!");
        }

        //判断token格式
        try {
            JwtUtils.parseToken(token);
        }catch (Exception e){
            return GatewayUtils.errorResponse(exchange,"token格式有误");
        }
        //有效性验证
        String userKey = JwtUtils.getUserKey(token);
        if (!redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey)){
            return GatewayUtils.errorResponse(exchange,"token过期!");
        }else{
            Long expire = redisTemplate.getExpire(TokenConstants.LOGIN_TOKEN_KEY + userKey, TimeUnit.MINUTES);

            if (expire<=10){
                redisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY+userKey,10,TimeUnit.MINUTES);
            }
        }
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
