package com.bwie.job.controller;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Job;
import com.bwie.common.domain.Log;
import com.bwie.common.domain.User;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.job.service.impl.JobServiceImpl;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@RestController
public class JobController {
    @Autowired
    private JobServiceImpl jobService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    //列表展示
    @PostMapping("/findJobList")
    public Result<List<Job>>findJobList(@RequestBody Job job){
        List<Job> jobList = jobService.findJobList( job.getJobName(), job.getFirmName() );
        return Result.success(jobList);
    }
    //投简历
    @PostMapping("/updjobStatusById")
    public Result updjobStatusById(@RequestBody Job job){

        Integer i = jobService.updjobStatusById( job.getFirmId() );
//        当前登录人信息
        String token = request.getHeader( "token" );
        String userKey = JwtUtils.getUserKey( token );
        String s = redisTemplate.opsForValue().get( "token:" + userKey );
        User user = JSONObject.parseObject( s, User.class );


//        判断是否投成功
        if(i<0){
            return Result.error(500,"未成功 请重试");
        }
//log_id主键 //job_name岗位名称 //user_name候选人名称 //user_tel候选人电话 //expect_position	期望岗位
// expect_money	期望薪资  //time投递时间 //log_status	状态（0为待审核 1为已通过 2为不通过）
        //判断 如果已经投过简历了

        if(jobService.findByUserTel( user.getUserTel() )!=null){
            return Result.error(500,"该岗位您已投递过，请耐心等待！");
        }
        Log log = new Log();
        log.setJobName( job.getJobName() );
        log.setUserName( "baseDao" );
        log.setUserTel( user.getUserTel() );
        log.setExpectPosition( job.getJobName() );
        log.setExpectMoney( job.getJobMoney() );
        //设置投简历人唯一标识
        jobService.updJobUser( job.getFirmId(),user.getUserTel() );
//        添加至候选人表
        jobService.addLog( log );
//        人数加1
        jobService.updJobCount( job.getFirmId() );

        return Result.success(job);
    }

    @GetMapping("/findLogList")
    public Result<List<Log>> findLogList(){
        List<Log> logList = jobService.findLogList();
        return Result.success(logList);
    }

    //通过
    @PostMapping("/updLogPassByLogid")
    public Result updLogPassByLogid(@RequestParam Integer logId){
        jobService.updLogPassByLogid(logId);
//        rabbitTemplate.convertAndSend( "pass",message -> {
//            return message;
//        } );
        return Result.success("已通过");
    }
    //不通过
    @PostMapping("/updLogNotPassByLogid")
    public Result updLogNotPassByLogid(@RequestParam Integer logId){
        jobService.updLogNotPassByLogid(logId);
//        rabbitTemplate.convertAndSend( "notpass",message -> {
//            return message;
//        } );
        return Result.success("已驳回");
    }

}
